//
//  AnalogSynthesizerViewController.m
//  AnalogSynthesizer
//
//  Created by 成田 拓郎 on 10/05/23.
//  Copyright __MyCompanyName__ 2010. All rights reserved.
//

#import "AnalogSynthesizerViewController.h"

@implementation AnalogSynthesizerViewController

@synthesize keyboardView;
@synthesize volume;

@synthesize synthesizerOutput;

/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/

//オーディオセッション　割り込みリスナーコールバック関数
static void interruptionListenerCallback(void *inUserData, UInt32 interruptionState)
{
	AnalogSynthesizerViewController *controller = (AnalogSynthesizerViewController *)inUserData;
	
	if (interruptionState == kAudioSessionBeginInterruption)
	{
		//オーディオセッション割り込みの開始
		//再生の停止
		controller.synthesizerOutput.play = NO;
	}
	else if (interruptionState == kAudioSessionEndInterruption)
	{
		//オーディオセッション割り込みの終了
		//オーディオセッションのカテゴリを設定
		UInt32 sessionCategory = kAudioSessionCategory_SoloAmbientSound;
		AudioSessionSetProperty(kAudioSessionProperty_AudioCategory,
								sizeof(sessionCategory),
								&sessionCategory);
		//オーディオセッションのアクティブ化
		AudioSessionSetActive(true);
		//再生の再開
		controller.synthesizerOutput.play = YES;
	}
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];

	[keyboardView setAction:@selector(keyboardChange:)];
	[keyboardView setTarget:self];
	[volume setAction:@selector(volumeChange:)];
	[volume setTarget:self];

	//AudioUnitの準備
	SynthesizerOutput *synthe = [[SynthesizerOutput alloc] init];
	self.synthesizerOutput = synthe;
	[synthe release];

	//オーディオセッションを初期化
	AudioSessionInitialize(NULL, NULL, interruptionListenerCallback, self);
	//オーディオセッションのカテゴリを設定
	UInt32 sessionCategory = kAudioSessionCategory_SoloAmbientSound;
	AudioSessionSetProperty(kAudioSessionProperty_AudioCategory,
							sizeof(sessionCategory),
							&sessionCategory);
	//オーディオセッションのアクティブ化
	AudioSessionSetActive(true);
	// 再生開始
	self.synthesizerOutput.play = YES;
}

// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

- (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
	if (toInterfaceOrientation == UIInterfaceOrientationPortrait ||
		toInterfaceOrientation == UIInterfaceOrientationPortraitUpsideDown) {
		// ポートレート モード
		self.volume.center = CGPointMake(52, 89);
		self.volume.bounds = CGRectMake(0, 0, 48, 48);
	}
	else {
		// ランドスケープ モード
		self.volume.center = CGPointMake(428, 89);
		self.volume.bounds = CGRectMake(0, 0, 48, 48);
	}
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
	self.keyboardView = nil;
	self.volume = nil;
}


- (void)dealloc {
	[keyboardView release];
	[volume release];
	[synthesizerOutput release];
    [super dealloc];
}

-(void)keyboardChange:(id)sender {
	NSLog(@"Keyboard Message");
	synthesizerOutput.keyboard = keyboardView.keyDown;
}

-(void)volumeChange:(id)sender {
	NSLog(@"Volume Message");
}

@end
