//
//  AudioOutput.m
//
//  Created by 成田 拓郎 on 2010/02/09 - 2012/06/14.
//  Copyright 2009-2012 lovelyshark. All rights reserved.
//

#import "AudioOutput.h"

@implementation AudioOutput

//playプロパティ
-(BOOL)play
{
	return play;
}
-(void)isPlay:(BOOL)newPlay
{
	if (play == newPlay)
		return;
	play = newPlay;
	if (play)
		AudioOutputUnitStart(outputUnit); //RemoteIOスタート
	else
		AudioOutputUnitStop(outputUnit);  //RemoteIOストップ
}

//サウンド コールバック関数(クラス メンバ)
-(OSStatus)outputCallback:(AudioUnitRenderActionFlags *)ioActionFlags
			  inTimeStamp:(const AudioTimeStamp *)inTimeStamp
			  inBusNumber:(UInt32)inBusNumber
		   inNumberFrames:(UInt32)inNumberFrames
				   ioData:(AudioBufferList *)ioData
{
	OSStatus err = noErr;
	
//	NSLog(@"mNumberBuffers = %i mNumberChannels = %i", ioData->mNumberBuffers, ioData->mBuffers[0].mNumberChannels);

	return err;
}

//サウンド コールバック関数(本体)
static OSStatus OutputCallback(void                        *inRefCon,
							   AudioUnitRenderActionFlags  *ioActionFlags,
							   const AudioTimeStamp        *inTimeStamp,
							   UInt32                      inBusNumber,
							   UInt32                      inNumberFrames,
							   AudioBufferList             *ioData)
{
	OSStatus err = noErr;
	NSAutoreleasePool *arp = [[NSAutoreleasePool alloc] init];

	err = [(id)inRefCon outputCallback:ioActionFlags
						   inTimeStamp:inTimeStamp
						   inBusNumber:inBusNumber
						inNumberFrames:inNumberFrames
								ioData:ioData];

	[arp release];
	return err;
}

//AudioUint初期化
- (void)setupOutputUnit
{
	//呼び出すAudioUnit情報
	AudioComponentDescription desc;
	desc.componentType         = kAudioUnitType_Output;
	desc.componentSubType      = kAudioUnitSubType_RemoteIO;
	desc.componentManufacturer = kAudioUnitManufacturer_Apple;
	desc.componentFlags        = 0;
	desc.componentFlagsMask    = 0;

	//AudioComponet取得
	AudioComponent component;
	component = AudioComponentFindNext(NULL, &desc);

	//AudioUnit取得
	AudioComponentInstanceNew(component, &outputUnit);

	//AudioUnit初期化
	AudioUnitInitialize(outputUnit);

	//コールバック関数情報
	AURenderCallbackStruct callback;
	callback.inputProc       = OutputCallback;
	callback.inputProcRefCon = self;

	//コールバック関数の設定
	AudioUnitSetProperty(outputUnit,
						 kAudioUnitProperty_SetRenderCallback,
						 kAudioUnitScope_Global,
						 0,
						 &callback,
						 sizeof(AURenderCallbackStruct));

	//オーディオデータフォーマット設定
	outputFormat.mSampleRate        = SAMPLE_RATE;
    outputFormat.mFormatID          = kAudioFormatLinearPCM;
	outputFormat.mFormatFlags       = kAudioFormatFlagsCanonical;
    outputFormat.mBitsPerChannel    = sizeof(AudioSampleType) * 8;
    outputFormat.mChannelsPerFrame  = 2;
    outputFormat.mFramesPerPacket   = 1;
    outputFormat.mBytesPerFrame     = outputFormat.mBitsPerChannel / 8 * outputFormat.mChannelsPerFrame;
	outputFormat.mBytesPerPacket    = outputFormat.mBytesPerFrame * outputFormat.mFramesPerPacket;
	outputFormat.mReserved          = 0;
	UInt32 size = sizeof(outputFormat);
    OSStatus err;
	err = AudioUnitSetProperty(outputUnit,
                               kAudioUnitProperty_StreamFormat,
                               kAudioUnitScope_Input,
                               0,
                               &outputFormat,
                               size);
    NSLog(@"AudioUnitSetProperty = %ld", err);
    
	//オーディオデータフォーマット取得
    //	UInt32 size = sizeof(outputFormat);
	AudioUnitGetProperty(outputUnit,
						 kAudioUnitProperty_StreamFormat,
						 kAudioUnitScope_Input,
						 0,
						 &outputFormat,
						 &size);

#if defined(DEBUG)
	//ログ出力（デバッグ用）
	NSLog(@"samplerate = %f", outputFormat.mSampleRate);
	NSLog(@"bits       = %lu", outputFormat.mBitsPerChannel);
	NSLog(@"channels   = %lu", outputFormat.mChannelsPerFrame);
	NSLog(@"%@",
		  (outputFormat.mFormatFlags & kLinearPCMFormatFlagIsNonInterleaved) ?
		  @"buffer     = non interleaved" : @"buffer     = interleaved");
	if (outputFormat.mFormatFlags & kLinearPCMFormatFlagIsFloat)
		NSLog(@"data type  = float");
	if (outputFormat.mFormatFlags & kLinearPCMFormatFlagIsSignedInteger)
		NSLog(@"data type  = single integer");
#endif

	//RemoteIOスタート
//	AudioOutputUnitStart(outputUnit);
//	play = YES;
	play = NO;
}

//AudioUnit終了処理
- (void)dispose
{
	if (play)
		AudioOutputUnitStop(outputUnit);
	AudioUnitUninitialize(outputUnit);
	AudioComponentInstanceDispose(outputUnit);
	outputUnit = NULL;
}

//初期化
- (id)init
{
	self = [super init];
	if (self != nil)
	{
		//AudioUint初期化
		[self setupOutputUnit];
	}
	return self;
}

//終了処理
-(void)dealloc
{
	[self dispose];
	[super dealloc];
}

@end
